/*
 QFeynCADCombined.ui.h
 Copyright (C) 1999-2004 Paolo Ribeca

 This file is part of FeynCAD package.

 FeynCAD is free software;
 you can redistribute it and/or modify it under the terms
 of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with FeynCAD; see the file COPYING.
 If not, write to the Free Software Foundation,
 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

void QFeynCADCombined::init() {
  pair<int,int> gridSize=this->_editor->_status.gridSize;
  this->SpinBox1->setValue(gridSize.first);
  this->SpinBox2->setValue(gridSize.second);
  this->_progressTimer=new QTimer(this);
  //
  connect(this->_editor,SIGNAL(newGridPosition(int,int)),
	  this,SLOT(updateGridPos(int,int)));
  connect(this->_editor,SIGNAL(vertexListUpdateReq(const QStringList&)),
          this,SLOT(updateVertexList(const QStringList&)));
  connect(this->_editor,SIGNAL(lineListUpdateReq(const QStringList&)),
          this,SLOT(updateLineList(const QStringList&)));
  connect(this->_editor,SIGNAL(tagListUpdateReq(const QStringList&)),
          this,SLOT(updateTagList(const QStringList&)));
  connect(this->ListBox1,SIGNAL(selected(int)),
          this->_editor,SLOT(selectVertexWhenListIs(int)));
  connect(this->ListBox2,SIGNAL(selected(int)),
          this->_editor,SLOT(selectLineWhenListIs(int)));
  connect(this->ListBox3,SIGNAL(selected(int)),
          this->_editor,SLOT(selectTagWhenListIs(int)));
  // Update del viewer: questo forza anche automaticamente i due widget
  //  ad avere la stessa taglia
  connect(this->_editor,SIGNAL(viewerUpdateReq(unsigned,unsigned)),
	  this->_viewer,SLOT(redrawPs(unsigned,unsigned)));
  connect(this->_editor,SIGNAL(gridChanged(unsigned,unsigned)),
	  this,SLOT(setGridTo(unsigned,unsigned)));
  // Sincronizzazione della status bar
  connect(this->_progressTimer,SIGNAL(timeout()),
	  this,SLOT(updateProgress()));
  connect(this->_viewer,SIGNAL(pageStarted()),this,SLOT(startProgress()));
  connect(this->_viewer,SIGNAL(pageCompleted()),
	  this,SLOT(stopProgress()));
  connect(this->_viewer,SIGNAL(pageError()),this,SLOT(errorProgress()));
  connect(this->_viewer,SIGNAL(pageTimeout()),this,SLOT(timeoutProgress()));
  // Cambio di modo
  connect(this->ComboBox,SIGNAL(activated(int)),this,SLOT(setModeTo(int)));
  this->setModeTo(0);
  //
  return;
}

void QFeynCADCombined::updateGridPos(int x,int y) {
  this->LCDNumber1->display(x);
  this->LCDNumber2->display(y);
  return;
}

void QFeynCADCombined::startProgress() {
  this->Label1->setPaletteBackgroundColor(green);
  this->Label1->setText("Busy");
  this->_progressTimer->start(300);
  return;
}

void QFeynCADCombined::updateProgress() {
    if (this->Label1->text()==QString("Busy"))
	this->Label1->setText("...");
    else
	this->Label1->setText("Busy");
    return;
}

void QFeynCADCombined::stopProgress() {
  this->_progressTimer->stop();
  this->Label1->setText("");
  this->Label1->setPaletteBackgroundColor(white);
  return;
}

void QFeynCADCombined::errorProgress() {
  this->_progressTimer->stop();
  this->Label1->setPaletteBackgroundColor(red);
  this->Label1->setText("Error");
  return;
}

void QFeynCADCombined::timeoutProgress() {
    this->_progressTimer->stop();
    QColor orange;
    orange.setNamedColor("Orange");
    this->Label1->setPaletteBackgroundColor(orange);
    this->Label1->setText("Timeout");
    ostringstream message;
    message << "Specified file\n\"" << this->_viewer->fileName()
	    << "\"\ncould not render itself within the specified time of "
	    << QFeynCADEditor::_timeout << " second"
	    << (QFeynCADEditor::_timeout==1?"":"s") << ":\n"
	    << "process displaying it has been terminated.\n"
	    << "Please choose a longer timeout in menu\n"
	    << "\"Edit->Global settings->Paths & Commands->Rendering timeout\"\n"
	    << "if you wish to alter this behaviour.";
    QMessageBox::warning(this,"QFeynCADCombined",message.str().c_str());
    return;
}

void QFeynCADCombined::setBaseFileNameTo(string fileName) {
  this->_baseFileName=fileName;
  ostringstream psFileName;
  psFileName << this->_baseFileName << ".eps";
  this->_viewer->setFileTo(psFileName.str().c_str());
  // Qui c'e' compreso l'update del viewer
  this->_editor->setFileNameTo(this->_baseFileName);
  return;
}

void QFeynCADCombined::updateVertexList(const QStringList& vertexList) {
  this->ListBox1->clear();
  ostringstream vertexNum;
  vertexNum << vertexList.size() << " "
            << (vertexList.size()==1?"Vertex":"Vertices");
  this->ToolBox1->setItemLabel(0,vertexNum.str().c_str());  
  this->ListBox1->insertStringList(vertexList);
  return; 
}

void QFeynCADCombined::updateLineList(const QStringList & lineList) {
  this->ListBox2->clear();
  ostringstream lineNum;
  lineNum << lineList.size() << " "
	  << (lineList.size()==1?"Line Group":"Line Groups");
  this->ToolBox1->setItemLabel(1,lineNum.str().c_str());
  this->ListBox2->insertStringList(lineList);
  return; 
}

void QFeynCADCombined::updateTagList(const QStringList & tagList) {
  this->ListBox3->clear();
  ostringstream tagNum;
  tagNum << tagList.size() << " "
	 << (tagList.size()==1?"Tag":"Tags");
  this->ToolBox1->setItemLabel(2,tagNum.str().c_str());  
  this->ListBox3->insertStringList(tagList);
  return; 
}

void QFeynCADCombined::setGridTo( unsigned x, unsigned y ) {
  this->SpinBox1->setValue(x);
  this->SpinBox2->setValue(y);
  //
  this->updateGeometry();
  this->adjustSize();
  emit sizeChanged();
  return;
}

void QFeynCADCombined::setModeTo(int mode) {
  switch (mode) {
  case 0:
      this->_editor->setModeTo(QFeynCADEditor::DrawMode);
      this->ListBox1->setEnabled(true);
      this->ListBox2->setEnabled(true);
      this->ListBox3->setEnabled(true);
      break;
  case 1:
      this->_editor->setModeTo(QFeynCADEditor::SelectMode);
      this->ListBox1->setEnabled(false);
      this->ListBox2->setEnabled(false);
      this->ListBox3->setEnabled(false);
      break;
  }
  return;
}

void QFeynCADCombined::resizeGrid() {
  this->PushButtonApply->setEnabled(false);
  // Qui c'e' compreso l'update del viewer
  this->_editor->
	  setGridSizeTo(this->SpinBox1->value(),this->SpinBox2->value());
  return;
}

void QFeynCADCombined::startStopResizeGrid() {
  if (!this->GroupBoxGrid->isEnabled()) {
      this->PushButtonEdit->setText("Back");
      this->PushButtonApply->setEnabled(false);
      this->Frame1->setEnabled(false);
      this->Frame2->setEnabled(false);
      this->Frame3->setEnabled(false);
      this->ComboBox->setEnabled(false);
      this->GroupBoxGrid->setEnabled(true);
  } else {
      this->PushButtonEdit->setText("Edit");
      this->SpinBox1->clearFocus();
      this->SpinBox2->clearFocus();
      this->Frame1->setEnabled(true);
      this->Frame2->setEnabled(true);
      this->Frame3->setEnabled(true);
      this->ComboBox->setEnabled(true);
      this->GroupBoxGrid->setEnabled(false);
  }
  return;
}

void QFeynCADCombined::activateApply() {
  if (!this->PushButtonApply->isEnabled())
      this->PushButtonApply->setEnabled(true);
  return;
}
