/*
 QFeynCADLineEdit.ui.h
 Copyright (C) 1999-2004 Paolo Ribeca

 This file is part of FeynCAD package.

 FeynCAD is free software;
 you can redistribute it and/or modify it under the terms
 of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with FeynCAD; see the file COPYING.
 If not, write to the Free Software Foundation,
 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

void QFeynCADLineEdit::init() {
  QRegExp regExpPosFloat("\\d{1,4}(\\.\\d{0,4})?|\\.\\d{1,4}");
  QValidator* validator2=new QRegExpValidator(regExpPosFloat,this->LineEdit2);
  QValidator* validator4=new QRegExpValidator(regExpPosFloat,this->LineEdit4);
  QValidator* validator7=new QRegExpValidator(regExpPosFloat,this->LineEdit7);
  this->LineEdit2->setValidator(validator2);
  this->LineEdit4->setValidator(validator4);
  this->LineEdit7->setValidator(validator7);
  QRegExp regExpFloat("(\\+|\\-)?(\\d{1,4}(\\.\\d{0,4})?|\\.\\d{1,4})");
  QValidator* validator5=new QRegExpValidator(regExpFloat,this->LineEdit5);
  QValidator* validator6=new QRegExpValidator(regExpFloat,this->LineEdit6);
  this->LineEdit5->setValidator(validator5);
  this->LineEdit6->setValidator(validator6);
  return;
}

line QFeynCADLineEdit::read() {
  this->normalize();
  return line(make_pair(this->TextLabel1->text().toInt(),
			this->TextLabel2->text().toInt()),
	      make_pair(this->TextLabel3->text().toInt(),
			this->TextLabel4->text().toInt()),
	      this->LineEdit1->text().ascii(),
	      this->LineEdit5->text()==""?-361:this->LineEdit5->text().toFloat(),
	      this->LineEdit6->text()==""?-361:this->LineEdit6->text().toFloat(),
	      this->LineEdit4->text()==""?1:this->LineEdit4->text().toFloat(),
	      this->LineEdit2->text().toFloat(),this->LineEdit3->text().ascii(),
	      this->LineEdit7->text()==""?1:this->LineEdit7->text().toFloat(),
	      this->SpinBox1->value());
}

void QFeynCADLineEdit::setTo( const line & l ) {
  this->TextLabel1->setNum(l.vOne.first);
  this->TextLabel2->setNum(l.vOne.second);
  this->TextLabel3->setNum(l.vTwo.first);
  this->TextLabel4->setNum(l.vTwo.second);
  this->LineEdit1->setText(l.type.c_str());
  this->LineEdit2->setText(QString::number(l.size));
  this->LineEdit3->setText(l.color.c_str());
  this->LineEdit5->setText(l.angleOne==-361?"":QString::number(l.angleOne));
  this->LineEdit6->setText(l.angleTwo==-361?"":QString::number(l.angleTwo));
  this->LineEdit4->setText(l.tension==1?"":QString::number(l.tension));
  this->LineEdit7->setText(l.rubout==1?"":QString::number(l.rubout));
  this->SpinBox1->setValue(l.depth);
  //
  this->normalize();
  // Si unsetta il delete
  this->PushButton2->setText("Delete");
  this->MainGroupBox->setEnabled(true);
  this->PushButton1->setEnabled(true);
  return;
}

void QFeynCADLineEdit::toggleDeleteUndelete() {
  if (this->PushButton2->text()=="Delete") {
      this->PushButton2->setText("Undelete");
      this->MainGroupBox->setEnabled(false);
      this->PushButton1->setEnabled(false);
  } else {
      this->PushButton2->setText("Delete");
      this->MainGroupBox->setEnabled(true);
      this->PushButton1->setEnabled(true);
  }
  return;
}

bool QFeynCADLineEdit::isDeleted()
{ return (this->PushButton2->text()=="Undelete"); }

void QFeynCADLineEdit::reset() {
  line newLine(make_pair(this->TextLabel1->text().toInt(),
			 this->TextLabel2->text().toInt()),
	       make_pair(this->TextLabel3->text().toInt(),
			 this->TextLabel4->text().toInt()));
  this->setTo(newLine);
  return;
}

void QFeynCADLineEdit::invert() {
  line newLine=this->read();
  pair<int,int> pointSwap=newLine.vOne;
  newLine.vOne=newLine.vTwo;
  newLine.vTwo=pointSwap;
  float angleSwap=newLine.angleOne;
  newLine.angleOne=(newLine.angleTwo==-361?-361:newLine.angleTwo+180);
  if (newLine.angleOne>=360)
      newLine.angleOne-=360;
  if (newLine.angleOne>=180)
      newLine.angleOne-=360;
  newLine.angleTwo=(angleSwap==-361?-361:angleSwap+180);
  if (newLine.angleTwo>=360)
      newLine.angleTwo-=360;
  if (newLine.angleTwo>=180)
      newLine.angleTwo-=360;
  // Patch for the case -0
  if ((-newLine.angleOne)==0)
      newLine.angleOne=0;
  if ((-newLine.angleTwo)==0)
      newLine.angleTwo=0;
  this->setTo(newLine);
  return;
}

void QFeynCADLineEdit::normalize() {
  // Qui il key concept e' appunto quello di tenere tutte le stringhe
  //  normalizzate, cioe' di avere come valori possibili il default e
  //  un valore che rientra nel range accettabile
  pair<QString,string> res;
  string message;
  this->LineEdit2->setText((res=revalidate(FEYNCAD_THIN_REVALIDATOR,
				"Size",this->LineEdit2->text())).first);
  message+=res.second;
  this->LineEdit4->setText((res=revalidate(FEYNCAD_TENSION_REVALIDATOR,
				"Tension",this->LineEdit4->text())).first);
  message+=res.second;
  this->LineEdit5->setText((res=revalidate(FEYNCAD_LINE_ANGLE_REVALIDATOR,
				"Starting angle",this->LineEdit5->text())).first);
  message+=res.second;
  this->LineEdit6->setText((res=revalidate(FEYNCAD_LINE_ANGLE_REVALIDATOR,
				"Ending angle",this->LineEdit6->text())).first);
  message+=res.second;
  this->LineEdit7->setText((res=revalidate(FEYNCAD_RUBOUT_REVALIDATOR,
				"Rubout",this->LineEdit7->text())).first);
  message+=res.second;
  if (message.size())
      QMessageBox::information(this,"QFeynCADLineEditor",message.c_str(),
			       "Accept");
  emit normalized();
  return;
}

// Questa indegna stronzata e' dovuta al fatto che designer
//  non permette di connettere segnali a segnali
void QFeynCADLineEdit::emitActivateApplyReq()
{ emit activateApplyReq(); return; }

