/*
 QFeynCADMain.ui.h
 Copyright (C) 1999-2004 Paolo Ribeca

 This file is part of FeynCAD package.

 FeynCAD is free software;
 you can redistribute it and/or modify it under the terms
 of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with FeynCAD; see the file COPYING.
 If not, write to the Free Software Foundation,
 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

void QFeynCADMain::init() {
  this->FileDialog=
    new QFileDialog(QDir::currentDirPath(),QString::null,0,0,true);
  this->ConfigEdit=new QFeynCADConfigEdit(this);
  this->Infos=new QFeynCADInfos(0);
  this->Help_=new QFeynCADHelp(0);
  this->SignatureEdit=new QFeynCADSignatureEdit(0);
  this->accelerator=new QAccel(this);
  this->advanced=new QPopupMenu(this->Help);
  this->Help->insertItem("Advanced",this->advanced,-1,5);
  this->advancedTitleOne=new QLabel(this->advanced);
  this->advancedTitleTwo=new QLabel(this->advanced);
  this->advancedTitleThr=new QLabel(this->advanced);
  QFont advancedTitleFont(this->advancedTitleOne->font());
  advancedTitleFont.setBold(true);
  this->advancedTitleOne->setFont(advancedTitleFont);
  this->advancedTitleTwo->setFont(advancedTitleFont);
  this->advancedTitleThr->setFont(advancedTitleFont);
  this->advancedTitleOne->setText("Vertices: Type");
  this->advancedTitleTwo->setText("Vertices: Fill");
  this->advancedTitleThr->setText("Lines");
  this->advanced->insertItem(this->advancedTitleOne);
  this->advancedVerticesPolysAction->addTo(this->advanced);
  this->advancedVerticesEllipsesRectanglesAction->addTo(this->advanced);
  this->advanced->insertItem(this->advancedTitleTwo);
  this->advancedVerticesHalftonesAction->addTo(this->advanced);
  this->advancedVerticesTilingsAction->addTo(this->advanced);
  this->advanced->insertItem(this->advancedTitleThr);
  this->advancedLinesStylesAction->addTo(this->advanced);
  this->advancedLinesRuboutAction->addTo(this->advanced);
  // Accelerators other than the automatic ones from actions;
  //  the first and second one will be occasionally deactivated/reactivated
  this->accelerator->connectItem(this->accelerator->insertItem(Key_Space,0),
				 this,SLOT(editLast()));
  this->accelerator->connectItem(this->accelerator->insertItem(Key_Escape,1),
				 this,SLOT(fileExit()));
  this->accelerator->connectItem(this->accelerator->insertItem(Key_PageUp),
				 this,SLOT(editUndo()));
  this->accelerator->connectItem(this->accelerator->insertItem(Key_PageDown),
				 this,SLOT(editRedo()));
  this->accelerator->connectItem(this->accelerator->insertItem(Key_Insert),
				 this,SLOT(editPaste()));
  this->accelerator->connectItem(this->accelerator->insertItem(Key_Delete),
				 this,SLOT(editDelete()));
  this->accelerator->connectItem(this->accelerator->insertItem(Key_F1),
				 this,SLOT(toggleMode()));
  // There is no selection at the beginning
  this->editTransformAction->setEnabled(false);
  //  Automatic: // this->accelerator->setItemEnabled(0,true);
  this->editPasteAction->setEnabled(false);
  this->editDeleteAction->setEnabled(false);
  //
  this->Graph->setCheckable(true);
  connect(this->Graph,SIGNAL(activated(int)),this,SLOT(onPageRequested(int)));
  //
  connect(this->ConfigEdit,SIGNAL(gridSettingsChanged()),
	  this,SLOT(doUpdateGridsAndResize()));
  connect(this->ConfigEdit,SIGNAL(displayMessageReq(bool,string)),
	  this,SLOT(displayMessage(bool,string)));
  this->fileSignatureAction->setEnabled(false);
  connect(this->SignatureEdit,SIGNAL(signatureModified(QString,QString)),
	  this,SLOT(modifySignature(QString,QString)));
  this->fileCloseAction->setEnabled(false);
  return;
}

void QFeynCADMain::fileNew() {
  page newPage(this->TabWidget);
  // Si ridimensiona tutto ogni volta che cambia la griglia
  connect(newPage.widget(),SIGNAL(sizeChanged()),
	  this,SLOT(doResize()));
  connect(newPage.widget()->_editor,SIGNAL(setMainAppEnabledReq(bool)),
	  this,SLOT(setAppEnabled(bool)));
  connect(newPage.widget()->_editor,SIGNAL(undoRedoUpdateReq(string,string)),
	  this,SLOT(setUndoRedo(string,string)));
  connect(newPage.widget()->_editor,SIGNAL(newMode()),
	  this,SLOT(updateSelectionRelatedActions()));
  connect(newPage.widget()->_editor,SIGNAL(newSelection()),
	  this,SLOT(updateSelectionRelatedActions()));
  connect(newPage.widget()->_editor,SIGNAL(titleUpdateReq()),
	  this,SLOT(updateTitle()));
  connect(newPage.widget()->_editor,SIGNAL(message(bool,string)),
	  this,SLOT(displayMessage(bool,string)));
  connect(newPage.widget()->_viewer,SIGNAL(message(bool,string)),
	  this,SLOT(displayMessage(bool,string)));
  // _Deve_ stare qui per l'aggiornamento di undo/redo
  this->pages.insert(make_pair(newPage.widget(),newPage));
  this->TabWidget->addTab(newPage.widget(),newPage.title().c_str());
  newPage.widget()->adjustSize();
  this->doResize();
  //newPage.widget()->show();
  // Questo triggera automaticamente anche l'aggiornamento di undo/redo  
  this->TabWidget->showPage(newPage.widget());
  if (this->pages.size()>1)
      this->fileCloseAction->setEnabled(true);
  ostringstream message;
  message << "New page \"" << newPage.title() << "\" opened.";
  this->displayMessage(false,message.str());
  return;
}

void QFeynCADMain::fileOpen()
{ this->doFileOpen(""); return; }

bool QFeynCADMain::doFileOpen(string fileName) {
  if (fileName=="") {
      // Qui non si propone nulla
      this->FileDialog->setSelection("");
      this->FileDialog->setMode(QFileDialog::ExistingFile);
      this->FileDialog->setFilter("FeynCAD Markup Files (*.fcd)");
      this->FileDialog->setViewMode(QFileDialog::Detail);
      if (this->FileDialog->exec()!=QDialog::Accepted)
	  return false;
      fileName=this->FileDialog->selectedFile().ascii();
  }
  // Si apre preventivamente una nuova tab
  this->fileNew();
  this->pages[this->TabWidget->currentPage()].setFileNameTo(fileName);
  if (!this->pages[this->TabWidget->currentPage()].
      widget()->_editor->readFromFile(fileName)) {
      QWidget* dying=this->TabWidget->currentPage();
      this->TabWidget->removePage(dying);
      this->pages.erase(this->pages.find(dying));
      //this->doResize();
      return false;
  } else {
      QTime start;
      start.start();
      while (start.elapsed()<((5+QFeynCADEditor::_timeout)*1000)&&
	     this->pages[this->TabWidget->currentPage()].widget()->
	     _progressTimer->isActive())
	  // Si aspetta che il rendering della pagina sia completo
	  qApp->processEvents();
      this->fileSignatureAction->setEnabled(true);
      ostringstream message;
      message << "Opened file \""
	      << this->pages[this->TabWidget->currentPage()].fileName()
	      << "\".";
      this->displayMessage(false,message.str());
      return true;
  }
}

void QFeynCADMain::fileSave() {
  if (this->pages[this->TabWidget->currentPage()].fileName()==string(""))
    { this->fileSaveAs(); return; }
  this->pages[this->TabWidget->currentPage()].widget()->_editor->
    dumpTo(this->pages[this->TabWidget->currentPage()].fileName());
  this->rebuildGraphMenu(this->TabWidget->currentPage());
  this->fileSignatureAction->setEnabled(true);
  ostringstream message;
  message << "Saved file \""
	  << this->pages[this->TabWidget->currentPage()].fileName() << "\".";
  this->displayMessage(false,message.str());
  return;
}

void QFeynCADMain::fileSaveAs() {
  // Qui non si propone niente
  this->FileDialog->setSelection("");
  this->FileDialog->setMode(QFileDialog::AnyFile);
  this->FileDialog->setFilter("FeynCAD Markup Files (*.fcd)");
  this->FileDialog->setViewMode(QFileDialog::Detail);
  if (this->FileDialog->exec()==QDialog::Accepted) {
      QString fileName=this->FileDialog->selectedFile();
      this->pages[this->TabWidget->currentPage()].
	      setFileNameTo(fileName.ascii());
      this->fileSave();
  }
  return;
}

void QFeynCADMain::fileClose() {
  // Si controlla che la tab corrente non sia l'unica
  if (this->pages.size()>1) {
    this->notYetSaved(this->pages[this->TabWidget->currentPage()],false);
    string memTitle=this->pages[this->TabWidget->currentPage()].fileName();
    QFeynCADCombined* mem=
	    this->pages[this->TabWidget->currentPage()].widget();
    this->pages.erase(this->pages.find(this->TabWidget->currentPage()));
    this->TabWidget->removePage(mem);
    delete mem;
    if (this->pages.size()==1)
	this->fileCloseAction->setEnabled(false);
    // Si riaggiusta la taglia generale
    this->doResize();
    ostringstream message;
    message << "Page \"" << memTitle << "\" closed.";
    this->displayMessage(false,message.str());
   } else
       this->fileCloseAction->setEnabled(false); // :)
  return;
}

void QFeynCADMain::fileExit() {
  // Controllo su eventuali file non salvati  
  for (map<QWidget*,page>::iterator
       i=this->pages.begin();i!=this->pages.end();++i)
      if (!this->notYetSaved(i->second,true))
	  return;
  qApp->quit();
  return;  
}

void QFeynCADMain::fileExportEPS() {
  // Si propongono il nome standard se la page corrente e' gia' stata salvata,
  //  oppure il nome standard derivato dall'ultimo file .fcd aperto
  QString fileName=
	  this->pages[this->TabWidget->currentPage()].fileName().c_str();
  if (fileName=="")
      fileName=this->FileDialog->selectedFile();
  if (fileName.endsWith(".fcd")) {
      fileName=fileName.replace(fileName.length()-4,4,".eps");
      this->FileDialog->setSelection(fileName);
  }
  this->FileDialog->setMode(QFileDialog::AnyFile);
  this->FileDialog->setFilter("Encapsulated Postscript Files (*.eps)");
  this->FileDialog->setViewMode(QFileDialog::Detail);
  if (this->FileDialog->exec()==QDialog::Accepted) {
    fileName=this->FileDialog->selectedFile();
    this->pages[this->TabWidget->currentPage()].widget()->_editor->
      exportEpsTo(this->pages[this->TabWidget->currentPage()].tmpFileName(),
		  fileName.ascii());
    ostringstream message;
    message << "Page \""
	    << this->pages[this->TabWidget->currentPage()].title(true)
	    << "\" exported to EPS file \"" << fileName.ascii() << "\".";
    this->displayMessage(false,message.str());
  }
  return;
}

void QFeynCADMain::fileSignature() {
  QFeynCADEditor* current=
	  this->pages[this->TabWidget->currentPage()].widget()->_editor;
  this->SignatureEdit->
	  setTo(this->pages[this->TabWidget->currentPage()].fileName(),
		current->_author,current->_creationDateTime,
		current->_modificationDateTime,current->_comments);
  this->SignatureEdit->PushButtonApply->setEnabled(false);
  this->SignatureEdit->show();
  return;
}

void QFeynCADMain::editUndo() {
  // Questo slot puo' essere chiamato direttamente da tastiera
  //  anche quando l'azione corrispondente e' disabilitata
  if (this->editUndoAction->isEnabled())
      this->pages[this->TabWidget->currentPage()].widget()->_editor->
	      goBackInHistory();
  return;
}

void QFeynCADMain::editRedo() {
  // Questo slot puo' essere chiamato direttamente da tastiera
  //  anche quando l'azione corrispondente e' disabilitata
  if (this->editRedoAction->isEnabled())
      this->pages[this->TabWidget->currentPage()].widget()->_editor->
	      goForwardInHistory();
  return;
}

void QFeynCADMain::editPaste() {
  // Questo slot puo' essere chiamato direttamente da tastiera
  //  anche quando l'azione corrispondente e' disabilitata
  if (this->editPasteAction->isEnabled()) {
      this->pages[this->TabWidget->currentPage()].widget()->_editor->
	      setModeTo(QFeynCADEditor::PasteMode);
      this->displayMessage(false,"Pasting current selection...");
  }
  return;
}

void QFeynCADMain::editLast() {
  this->pages[this->TabWidget->currentPage()].widget()->_editor->
	  editLast();
  return;
}

void QFeynCADMain::editDelete() {
  // Questo slot puo' essere chiamato direttamente da tastiera
  //  anche quando l'azione corrispondente e' disabilitata
  if (this->editDeleteAction->isEnabled()) {
      this->pages[this->TabWidget->currentPage()].widget()->_editor->
	      deleteSelection();
      this->editTransformAction->setEnabled(false);
      this->editDeleteAction->setEnabled(false);
      this->displayMessage(false,"Current selection deleted");
  }
  return;
}

void QFeynCADMain::editTransform() {
  QFeynCADEditor* current=
	  this->pages[this->TabWidget->currentPage()].widget()->_editor;
  // Si copia l'(_alpha) dell'editor in quello locale
  current->_modifySelection->_alpha=current->_alpha;
  current->_modifySelection->
	  actualBoundariesAre(0,current->_status.gridSize.first,
			      0,current->_status.gridSize.second);
  current->_modifySelection->newBoundariesAre
	  (min(current->_selRectOne.first,current->_selRectTwo.first),
	   max(current->_selRectOne.first,current->_selRectTwo.first),
	   min(current->_selRectOne.second,current->_selRectTwo.second),
	   max(current->_selRectOne.second,current->_selRectTwo.second));
  this->setAppEnabled(false);
  current->_modifySelection->setEnabled(true);
  current->_modifySelection->PushButton2->setEnabled(false);
  current->_modifySelection->show();
  current->_runawayDialogCatcher->start(100);
  ostringstream descr;
  descr << "Transforming Selection (" << current->_selRectOne.first << ","
	  << current->_selRectOne.second << ")<>(" << current->_selRectTwo.first
	  << "," << current->_selRectTwo.second << ")...";
  this->displayMessage(true,descr.str());
  return;
}

void QFeynCADMain::editGlobal() {
  this->ConfigEdit->update();
  this->ConfigEdit->PushButtonApply->setEnabled(false);
  this->ConfigEdit->show();
  return;
}

void QFeynCADMain::helpContents() { this->Help_->show(); return; }

#define INCLUDE_FCD_FILE(what) \
  this->fileNew();\
  this->pages[this->TabWidget->currentPage()].\
	  widget()->_editor->readFromBuffer(example);\
  ostringstream message;\
  message << "New page \""\
	  << this->pages[this->TabWidget->currentPage()].title()\
	  << "\" opened, with examples of "what".";\
  this->displayMessage(false,message.str());\
  //
  void QFeynCADMain::helpVertices() {
  string example(
#include "strings/examples/vertices.string"
	  );
  INCLUDE_FCD_FILE("vertices")
  return;
}
void QFeynCADMain::helpLines() {
  string example(
#include "strings/examples/lines.string"
	  );
  INCLUDE_FCD_FILE("lines")
  return;
}

void QFeynCADMain::helpTags() {
  string example(
#include "strings/examples/tags.string"
	  );
  INCLUDE_FCD_FILE("tags")
  return;
}

void QFeynCADMain::advancedLinesStyles() {
  string example(
#include "strings/examples/lineStyles.string"
	  );
  INCLUDE_FCD_FILE("line types")
  return;
}

void QFeynCADMain::advancedLinesRubout() {
  string example(
#include "strings/examples/lineRubout.string"
	  );
  INCLUDE_FCD_FILE("line rubout")
  return;
}

void QFeynCADMain::advancedVerticesPolys() {
  string example(
#include "strings/examples/vertexPolys.string"
	  );
  INCLUDE_FCD_FILE("poly- vertex types")
  return;
}

void QFeynCADMain::advancedVerticesTilings() {
  string example(
#include "strings/examples/vertexTilings.string"
	  );
  INCLUDE_FCD_FILE("tiled vertex fills")
  return;
}

void QFeynCADMain::advancedVerticesHalftones() {
  string example(
#include "strings/examples/vertexHalftones.string"
	  );
  INCLUDE_FCD_FILE("halftoned vertex fills")
  return;
}

void QFeynCADMain::advancedVerticesEllipsesRectangles() {
  string example(
#include "strings/examples/vertexEllipsesRectangles.string"
	  );
  INCLUDE_FCD_FILE("ellipse and rectangle vertex types")
  return;
}

void QFeynCADMain::helpInfos() { this->Infos->show(); return; }

void QFeynCADMain::setUndoRedo(string undo, string redo) {
  ostringstream undoText,redoText;
  undoText << "Undo last action";
  redoText << "Redo next action";
  if (undo!=string("")) {
    undoText << " \"" << undo << "\"";
    this->editUndoAction->setEnabled(true);
  }
  else
    this->editUndoAction->setEnabled(false);
  this->editUndoAction->setText(undoText.str().c_str());  
  this->editUndoAction->setMenuText(undoText.str().c_str());  
  this->editUndoAction->setToolTip(undoText.str().c_str());  
  if (redo!=string("")) {
    redoText << " \"" << redo << "\"";
    this->editRedoAction->setEnabled(true);
  }
  else
    this->editRedoAction->setEnabled(false);
  this->editRedoAction->setText(redoText.str().c_str());
  this->editRedoAction->setMenuText(redoText.str().c_str());
  this->editRedoAction->setToolTip(redoText.str().c_str());
  return;
}

void QFeynCADMain::rebuildGraphMenu(QWidget* visiblePage) {
  this->Graph->clear();
  register int id=0;
  for (map<QWidget*,page>::const_iterator
       i=this->pages.begin();i!=this->pages.end();++i) {
      this->Graph->insertItem(i->second.title().c_str(),id);
      this->Graph->setItemChecked(id,(visiblePage==i->first));  
      ++id;
  }
  return;
}

void QFeynCADMain::onPageChanged(QWidget *) {
  QFeynCADEditor* current=
	  this->pages[this->TabWidget->currentPage()].widget()->_editor;
  current->undoRedoUpdate();
  //
  this->rebuildGraphMenu(this->TabWidget->currentPage());
  // Si verifica se abilitare o no (Delete), (Transform) e (Paste)
  this->updateSelectionRelatedActions();
  // Si abilita (Signature) se alla pagina corrisponde un file
  this->fileSignatureAction->
	  setEnabled(!this->pages[this->TabWidget->currentPage()].isUntitled());
  return;
}

void QFeynCADMain::onPageRequested(int ind) {
  // In questo caso il cambio di pagina e' richiesto dal menu,
  //  non direttamente dal (tabWidget)
  map<QWidget*,page>::const_iterator where=this->pages.begin();
  for (register int i=0;i<ind;++i)
      ++where;
  this->TabWidget->showPage(where->first);
  return;
}

void QFeynCADMain::updateTitle() {
  // Si aggiorna il titolo del tab-widget
  this->TabWidget->changeTab(this->TabWidget->currentPage(),
			     this->pages[this->TabWidget->currentPage()].
			     title().c_str());
  //
  this->rebuildGraphMenu(this->TabWidget->currentPage());
  return;
}

bool QFeynCADMain::notYetSaved(page & p,bool exiting) {
  if (!p.widget()->_editor->_isUpToDate) {
    ostringstream message;
    message << "Page \"" << p.title(true)
	    << "\" contains unsaved changes that will be lost.\n"
	    << "Would you like to save them now?";
    switch(QMessageBox::warning(this,"QFeynCADEditor",
				message.str().c_str(),"I don't care: Delete Page",
				"Save Page",
				exiting?"Don't exit FeynCAD":QString::null,
				0,exiting?2:1)) {
    case 0:
	return true;
	break;
    case 1:
	// Si seleziona la tab in esame
	this->TabWidget->showPage(p.widget());
	if (p.fileName()==string(""))
	    this->fileSaveAs();
	else
	    this->fileSave();
	return true;
	break;
    default:
	return false;
	break;
    }
  }
  return true;
}

void QFeynCADMain::displayMessage(bool isPermanent, string m) {
  if (isPermanent)
    if (m!="")
      this->statusBar()->message(m.c_str());
    else
      this->statusBar()->clear();
  else
    this->statusBar()->message(m.c_str(),3000);
  return;
}

void QFeynCADMain::setAppEnabled(bool en) {
  this->MenuBar->setEnabled(en);
  this->GeneralToolBar->setEnabled(en);
  this->TabWidget->setEnabled(en);
  return;
}

void QFeynCADMain::doResize() {
  QSize hint;
  for (map<QWidget*,page>::iterator
       i=this->pages.begin();i!=this->pages.end();++i) {
      i->second.widget()->updateGeometry();
      i->second.widget()->adjustSize();
      QSize localHint=i->second.widget()->sizeHint();
      if (localHint.width()>hint.width()&&localHint.height()>hint.height())
	  hint=localHint;
  }
  // We have to force this
  this->TabWidget->resize(hint);
  //this->TabWidget->updateGeometry();
  //this->TabWidget->adjustSize();
  this->updateGeometry();  
  this->adjustSize();
  return;
}

void QFeynCADMain::doUpdateGridsAndResize() {
  QSize hint;
  for (map<QWidget*,page>::iterator
       i=this->pages.begin();i!=this->pages.end();++i) {
      pair<int,int> gridSize=i->second.widget()->_editor->_status.gridSize;
      i->second.widget()->_editor->
	      setGridSizeTo(gridSize.first,gridSize.second);
      i->second.widget()->updateGeometry();
      i->second.widget()->adjustSize();
      QSize localHint=i->second.widget()->sizeHint();
      if (localHint.width()>hint.width()&&localHint.height()>hint.height())
	  hint=localHint;
  }
  // We have to force this
  this->TabWidget->resize(hint);
  //this->TabWidget->updateGeometry();
  //this->TabWidget->adjustSize();
  this->updateGeometry();  
  this->adjustSize();
  return;
}

void QFeynCADMain::updateSelectionRelatedActions() {
  QFeynCADEditor* current=
	  this->pages[this->TabWidget->currentPage()].widget()->_editor;
  this->editPasteAction->
	  setEnabled((QFeynCADEditor::_clipboard.vertices.size()+
		      QFeynCADEditor::_clipboard.tags.size())&&
		     current->_mode==QFeynCADEditor::SelectMode);
  this->accelerator->setItemEnabled(0,!this->editPasteAction->isEnabled());
  this->accelerator->
    setItemEnabled(1,current->_mode!=QFeynCADEditor::PasteMode);
  bool isThereAnActiveSelection=(current->_selRectOne!=current->_selRectTwo&&
				 (current->_alpha.vertices.size()+
				  current->_alpha.tags.size()));
  this->editTransformAction->setEnabled(isThereAnActiveSelection);
  this->editDeleteAction->setEnabled(isThereAnActiveSelection);
  return;
}

void QFeynCADMain::modifySignature(QString author,QString comments) {
  this->SignatureEdit->PushButtonApply->setEnabled(false);
  QFeynCADEditor* current=
	  this->pages[this->TabWidget->currentPage()].widget()->_editor;
  current->_author=author;
  current->_comments=comments;
  current->_isUpToDate=false;
  this->updateTitle();
  ostringstream message;
  message << "Modified signature of file \""
	  << this->pages[this->TabWidget->currentPage()].fileName() << "\".";
  this->displayMessage(false,message.str());
  return;
}

void QFeynCADMain::toggleMode() {
  QFeynCADCombined* current=
	  this->pages[this->TabWidget->currentPage()].widget();
  switch (current->_editor->_mode) {
  case QFeynCADEditor::DrawMode:
      current->ComboBox->setCurrentItem(1);
      current->setModeTo(1);
      break;
  case QFeynCADEditor::SelectMode:
      current->ComboBox->setCurrentItem(0);
      current->setModeTo(0);
      break;
  default: break;
  }
  return;
}
