/*
 QFeynCADTagEdit.ui.h
 Copyright (C) 1999-2004 Paolo Ribeca

 This file is part of FeynCAD package.

 FeynCAD is free software;
 you can redistribute it and/or modify it under the terms
 of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with FeynCAD; see the file COPYING.
 If not, write to the Free Software Foundation,
 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

void QFeynCADTagEdit::init() {
  this->RadioButton0->setChecked(true);
  QRegExp regExpPos("\\d{1,4}(\\.\\d{0,4})?\\.\\d{1,4}");
  QValidator* validator4=new QRegExpValidator(regExpPos,this->LineEdit4);
  this->LineEdit4->setValidator(validator4);
  QRegExp regExp("(\\+|\\-)?(\\d{1,4}(\\.\\d{0,4})?|\\.\\d{1,4})");
  QValidator* validator5=new QRegExpValidator(regExp,this->LineEdit5);
  this->LineEdit5->setValidator(validator5);
  return;
}

tag QFeynCADTagEdit::read() {
  this->normalize();
  tag res(make_pair(this->SpinBox1->value(),this->SpinBox2->value()),
	  this->LineEdit1->text().ascii(),this->LineEdit2->text().ascii(),
	  this->LineEdit3->text().ascii(),
	  // Questo _non_ puo' essere completamente trappolato
	  this->LineEdit4->text().toFloat(),
	  this->LineEdit5->text()==""?0:this->LineEdit5->text().toFloat(),
	  "cc",this->SpinBox3->value());
  if (this->RadioButton0->isChecked()) res.controlPoint="cc";
  else if (this->RadioButton1->isChecked()) res.controlPoint="cr";
  else if (this->RadioButton2->isChecked()) res.controlPoint="tr";
  else if (this->RadioButton3->isChecked()) res.controlPoint="tc";
  else if (this->RadioButton4->isChecked()) res.controlPoint="tl";
  else if (this->RadioButton5->isChecked()) res.controlPoint="cl";
  else if (this->RadioButton6->isChecked()) res.controlPoint="bl";
  else if (this->RadioButton7->isChecked()) res.controlPoint="bc";
  else if (this->RadioButton8->isChecked()) res.controlPoint="br";
  return res;
}

void QFeynCADTagEdit::setTo( const tag & t ) {
  this->SpinBox1->setValue(t.pos.first);
  this->SpinBox2->setValue(t.pos.second);
  this->TextLabel1->setNum(t.pos.first);
  this->TextLabel2->setNum(t.pos.second);
  this->LineEdit1->setText(t.text.c_str());
  this->LineEdit2->setText(t.font.c_str());
  this->LineEdit3->setText(t.color.c_str());
  this->LineEdit4->setText(QString::number(t.scale));
  this->LineEdit5->setText(t.startingAngle?QString::number(t.startingAngle):"");
  this->SpinBox3->setValue(t.depth);
  if (t.controlPoint=="tl") this->RadioButton4->setChecked(true);
  else if (t.controlPoint=="tc") this->RadioButton3->setChecked(true);
  else if (t.controlPoint=="tr") this->RadioButton2->setChecked(true);
  else if (t.controlPoint=="cl") this->RadioButton5->setChecked(true);
  else if (t.controlPoint=="cr") this->RadioButton1->setChecked(true);
  else if (t.controlPoint=="bl") this->RadioButton6->setChecked(true);
  else if (t.controlPoint=="bc") this->RadioButton7->setChecked(true);
  else if (t.controlPoint=="br") this->RadioButton8->setChecked(true);
  //  Il caso "cc" fa da default implicito
  else this->RadioButton0->setChecked(true);
  //
  this->normalize();
  // Si unsetta il delete
  this->PushButton2->setText("Delete");
  this->MainGroupBox->setEnabled(true);
  this->PushButton1->setEnabled(true);
  return;
}

void QFeynCADTagEdit::toggleDeleteUndelete() {
  if (this->PushButton2->text()=="Delete") {
      this->PushButton2->setText("Undelete");
      this->MainGroupBox->setEnabled(false);
      this->PushButton1->setEnabled(false);
  } else {
      this->PushButton2->setText("Delete");
      this->MainGroupBox->setEnabled(true);
      this->PushButton1->setEnabled(true);
  }
  return;
}

bool QFeynCADTagEdit::isDeleted()
{ return (this->PushButton2->text()=="Undelete"); }

void QFeynCADTagEdit::reset() {
  tag newTag(make_pair(this->SpinBox1->value(),this->SpinBox2->value()));
  this->setTo(newTag);
  return;
}

void QFeynCADTagEdit::activateApply() {
  if (!this->PushButtonApply->isEnabled())
      this->PushButtonApply->setEnabled(true);
  return;
}

void QFeynCADTagEdit::normalize() {
  // Qui il key concept e' appunto quello di tenere tutte le stringhe
  //  normalizzate, cioe' di avere come valori possibili il default e
  //  un valore che rientra nel range accettabile
  pair<QString,string> res;
  string message;
  this->LineEdit4->setText((res=revalidate(FEYNCAD_SCALE_REVALIDATOR,
				"Scale",this->LineEdit4->text())).first);
  message+=res.second;
  this->LineEdit5->setText((res=revalidate(FEYNCAD_ANGLE_REVALIDATOR,
				"Starting angle",this->LineEdit5->text())).first);
  message+=res.second;
  if (message.size())
      QMessageBox::information(this,"QFeynCADTagEditor",message.c_str(),
			       "Accept");
  this->PushButtonApply->setEnabled(false);
  return;
}
