/*
 QFeynCADTransformEdit.ui.h
 Copyright (C) 1999-2004 Paolo Ribeca

 This file is part of FeynCAD package.

 FeynCAD is free software;
 you can redistribute it and/or modify it under the terms
 of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with FeynCAD; see the file COPYING.
 If not, write to the Free Software Foundation,
 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

void QFeynCADTransformEdit::init() {
  connect(this->PushButton1,SIGNAL(clicked()),
	  this,SIGNAL(clipToSelectionReq()));
  connect(this->PushButton2,SIGNAL(clicked()),
	  this,SIGNAL(acceptRotationReq()));
  return;
}

void QFeynCADTransformEdit::newBoundariesAre(int xl,int xr,int yb,int yt) {
  if (xl==-1&&xr==-1&&yb==-1&&yt==-1) {
    // Si resettano i buffer, e si riportano a zero i parametri di rotazione
    this->_alpha.clear();
    this->SpinBox1->setValue(0);
    this->SpinBox2->setValue(0);
    this->SpinBox3->setValue(0);
    //
    this->TextLabel1->setText("");
    this->TextLabel2->setText("");
    this->TextLabel3->setText("");
    this->TextLabel4->setText("");
  } else {
    this->TextLabel1->setNum(xl);
    this->TextLabel2->setNum(xr);
    this->TextLabel3->setNum(yb);
    this->TextLabel4->setNum(yt);
  }
  return;
}

void QFeynCADTransformEdit::actualBoundariesAre(int xl,int xr,int yb,int yt) {
  this->TextLabel5->setNum(xl);
  this->TextLabel6->setNum(xr);
  this->TextLabel7->setNum(yb);
  this->TextLabel8->setNum(yt);
  return;
}

void QFeynCADTransformEdit::realBoundingBoxIs(int xl,int xr,int yb,int yt) {
  this->TextLabel9->setNum(xl);
  this->TextLabel10->setNum(xr);
  this->TextLabel11->setNum(yb);
  this->TextLabel12->setNum(yt);
  return;
}

void QFeynCADTransformEdit::emitRotateAlphaReq() {
  int x=this->SpinBox2->value(),y=this->SpinBox3->value(),
  angle=this->SpinBox1->value();
  if (this->_alpha.size()) {
      if (!angle)
	  this->_transformedAlpha=this->_alpha;
      else {
	  // Si ruotano gli oggetti selezionati rispetto al punto (x,y)
	  //  Si crea un catalogo dei vertici da traslare a beneficio delle linee
	  map<pair<int,int>,pair<int,int> > rotatedVertices;
	  set<vertex> newVertices;
	  for (set<vertex>::iterator i=this->_alpha.vertices.begin();
	  i!=this->_alpha.vertices.end();++i) {
	      vertex newVertex(*i);
	      double mod=hypot(i->pos.first-x,i->pos.second-y),
	      startingAngle=atan2(double(i->pos.second-y),double(i->pos.first-x));
	      newVertex.pos.first=
		      x+(int)round(mod*cos(startingAngle-double(angle)/double(180)*M_PI));
	      newVertex.pos.second=
		      y+(int)round(mod*sin(startingAngle-double(angle)/double(180)*M_PI));
	      if (newVertex.startingAngle!=-361)
		  newVertex.startingAngle-=angle;
	      newVertices.insert(newVertex);
	      rotatedVertices.insert(make_pair(i->pos,newVertex.pos));
	  }
	  this->_transformedAlpha.vertices=newVertices;
	  multiset<line> newLines;
	  map<pair<int,int>,pair<int,int> >::const_iterator found;
	  for (multiset<line>::iterator
	       i=this->_alpha.lines.begin();i!=this->_alpha.lines.end();++i) {
	      line newLine(*i);
	      if ((found=rotatedVertices.find(i->vOne))!=rotatedVertices.end()) {
		  newLine.vOne=found->second;
		  if (newLine.angleOne!=-361)
		      newLine.angleOne-=angle;
	      }
	      if ((found=rotatedVertices.find(i->vTwo))!=rotatedVertices.end()) {
		  newLine.vTwo=found->second;
		  // Nel caso del tadpolo il secondo angolo non deve essere ruotato
		  if (newLine.angleTwo!=-361&&newLine.vOne!=newLine.vTwo)
		      newLine.angleTwo-=angle;
	      }
	      newLines.insert(newLine);
	  }
	  this->_transformedAlpha.lines=newLines;
	  set<tag> newTags;
	  for (set<tag>::iterator
	       i=this->_alpha.tags.begin();i!=this->_alpha.tags.end();++i) {
	      tag newTag(*i);
	      double mod=hypot(i->pos.first-x,i->pos.second-y),
	      startingAngle=atan2(double(i->pos.second-y),double(i->pos.first-x));
	      newTag.pos.first=
		      x+(int)round(mod*cos(startingAngle-double(angle)/double(180)*M_PI));
	      newTag.pos.second=
		      y+(int)round(mod*sin(startingAngle-double(angle)/double(180)*M_PI));
	      if (newTag.startingAngle!=-361)
		  newTag.startingAngle-=angle;
	      newTags.insert(newTag);
	  }
	  this->_transformedAlpha.tags=newTags;
      }
  }
  emit rotateAlphaReq(x,y,angle);
  return;
}

void QFeynCADTransformEdit::onPageChanged() {
  switch (TabWidget->currentPageIndex()) {
  case 0:
      // Clip menu
      //  Si annullano eventuali effetti di rotazioni precedenti
      this->_transformedAlpha=this->_alpha;
      //  Per fare sparire la cornice di selezione
      emit rotateAlphaReq(-1,-1,0);
      break;
  case 1:
      // Rotate menu
      //  Si aggiorna la situazione con i parametri di rotazione
      //   attualmente memorizzati
      this->emitRotateAlphaReq();
      break;
  default:
      break;
  }
  return;
}

void QFeynCADTransformEdit::activateApplyRotate() {
  if (!this->PushButton2->isEnabled())
      this->PushButton2->setEnabled(true);
  return;
}
