/*
 QFeynCADVertexEdit.ui.h
 Copyright (C) 1999-2004 Paolo Ribeca

 This file is part of FeynCAD package.

 FeynCAD is free software;
 you can redistribute it and/or modify it under the terms
 of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with FeynCAD; see the file COPYING.
 If not, write to the Free Software Foundation,
 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

void QFeynCADVertexEdit::init() {
  QRegExp regExpPos("\\d{1,4}(\\.\\d{0,4})?|\\.\\d{1,4}");
  QValidator* validator2=new QRegExpValidator(regExpPos,this->LineEdit2);
  this->LineEdit2->setValidator(validator2);
  QValidator* validator3=new QRegExpValidator(regExpPos,this->LineEdit3);
  this->LineEdit3->setValidator(validator3);
  QRegExp regExp("(\\+|\\-)?(\\d{1,4}(\\.\\d{0,4})?|\\.\\d{1,4})");
  QValidator* validator7=new QRegExpValidator(regExp,this->LineEdit7);
  this->LineEdit7->setValidator(validator7);
  return;
}

vertex QFeynCADVertexEdit::read() {
  this->normalize();
  return vertex(make_pair(this->SpinBox1->value(),this->SpinBox2->value()),
		this->LineEdit1->text().ascii(),
		this->LineEdit7->text()==""?0:this->LineEdit7->text().toFloat(),
		this->LineEdit2->text().toFloat(),this->LineEdit3->text().toFloat(),
		this->LineEdit6->text().ascii(),this->LineEdit4->text().ascii(),
		this->LineEdit5->text().ascii(),this->SpinBox3->value(),
		this->CheckBox1->isChecked());
}

void QFeynCADVertexEdit::setTo( const vertex& v ) {
  this->SpinBox1->setValue(v.pos.first);
  this->SpinBox2->setValue(v.pos.second);
  this->TextLabel1->setNum(v.pos.first);
  this->TextLabel2->setNum(v.pos.second);
  this->LineEdit1->setText(v.type.c_str());
  this->LineEdit2->setText(QString::number(v.size));
  this->LineEdit3->setText(QString::number(v.border));
  this->LineEdit4->setText(v.foreColor.c_str());
  this->LineEdit5->setText(v.backColor.c_str());
  this->LineEdit6->setText(v.fill.c_str());
  this->LineEdit7->setText(v.startingAngle?QString::number(v.startingAngle):"");
  this->SpinBox3->setValue(v.depth);
  this->CheckBox1->setChecked(v.isSticky);
  //
  this->normalize();
  // Si unsetta il delete
  this->PushButton2->setText("Delete");
  this->MainGroupBox->setEnabled(true);
  this->PushButton1->setEnabled(true);
  return;
}

void QFeynCADVertexEdit::toggleDeleteUndelete() {
  if (this->PushButton2->text()=="Delete") {
      this->PushButton2->setText("Undelete");
      this->MainGroupBox->setEnabled(false);
      this->PushButton1->setEnabled(false);
  } else {
      this->PushButton2->setText("Delete");
      this->MainGroupBox->setEnabled(true);
      this->PushButton1->setEnabled(true);
  }
  return;
}

bool QFeynCADVertexEdit::isDeleted()
{ return (this->PushButton2->text()=="Undelete"); }

void QFeynCADVertexEdit::reset() {
  vertex newVertex(make_pair(this->SpinBox1->value(),
			     this->SpinBox2->value()));
  this->setTo(newVertex);
  return;
}

void QFeynCADVertexEdit::activateApply() {
  if (!this->PushButtonApply->isEnabled())
      this->PushButtonApply->setEnabled(true);
  return;
}

void QFeynCADVertexEdit::normalize() {
  // Qui il key concept e' appunto quello di tenere tutte le stringhe
  //  normalizzate, cioe' di avere come valori possibili il default e
  //  un valore che rientra nel range accettabile
  pair<QString,string> res;
  string message;
  this->LineEdit2->setText((res=revalidate(FEYNCAD_THIN_REVALIDATOR,
				"Size",this->LineEdit2->text())).first);
  message+=res.second;
  this->LineEdit3->setText((res=revalidate(FEYNCAD_THIN_REVALIDATOR,
				"Border size",this->LineEdit3->text())).first);
  message+=res.second;
  this->LineEdit7->setText((res=revalidate(FEYNCAD_ANGLE_REVALIDATOR,
				"Starting angle",this->LineEdit7->text())).first);
  message+=res.second;
  if (message.size())
      QMessageBox::information(this,"QFeynCADVertexEditor",message.c_str(),
			       "Accept");
  this->PushButtonApply->setEnabled(false);
  return;
}
