// QPsWidget.h
// Copyright (C) 1999-2004 Paolo Ribeca

// This file is part of FeynCAD package.

// FeynCAD is free software;
// you can redistribute it and/or modify it under the terms
// of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with FeynCAD; see the file COPYING.
// If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

// Implementation of ghostscript-related handshaking is borrowed from
//  some 3.x.x versions of (kghostview).

#include <string>

#include <qwidget.h>
#include <qprocess.h>
#include <qpaintdevicemetrics.h>
#include <qpixmap.h>
#include <qspinbox.h>
#include <qlayout.h>
#include <qtimer.h>

#include <X11/Xlib.h>
#include <X11/Xatom.h>

using namespace std;

class QPsWidget:
  public QWidget
{
  Q_OBJECT
 private:
  enum AtomName { GHOSTVIEW = 0, GHOSTVIEW_COLORS, NEXT, PAGE, DONE };
  class boundingBox {
    int one;
    int two;
    int thr;
    int fou;
  public:
    void setTo(int iOne,int iTwo,int iThr,int iFou)
    { this->one=iOne,this->two=iTwo,this->thr=iThr,this->fou=iFou; }
    int width() { return this->thr-this->one; }
    int height() { return this->fou-this->two; }
    //
    friend class QPsWidget;
  };
 protected:
  QPixmap* _backgroundPixmap;
  // X-related stuff
  Window _gsWindow;   // Destination of ghostscript messages.
  Atom _atoms[5];
  //
  unsigned _requestedWidth;
  unsigned _requestedHeight;
  string _fileName;
  //
  pair<int,int> _logicalDpi;
  boundingBox _boundingBox;
  // Queste due quantita' sono derivate dalle precedenti
  float _xMagnification;
  float _yMagnification;
  //
  QProcess* _process;
  bool _isBusy;
  bool _isReqPending;
  //
  QTimer _timeoutTimer;
  //
  static string _ghostscriptPath;
  static unsigned _timeout;
 protected:
  bool startInterpreter();
  bool stopInterpreter();
  bool x11Event(XEvent* e);
 protected slots:
  void timeout();
  void outputErrors();
 public:
  QPsWidget(QWidget* parent=0,const char* name=0,
	    unsigned requestedWidth=0,unsigned requestedHeight=0,
	    string fileName="");
  ~QPsWidget();
  //
  inline string fileName() { return this->_fileName; }
 signals:
  void pageStarted();
  void pageError();
  void pageCompleted();
  void pageTimeout();
  void message(bool isPermanent,string m);
 public slots:
  // Il widget si comporta cosi':
  //  -1- se larghezza e altezza non sono specificate
  //      -1a- se il file e' specificato lo si visualizza a ingrandimento (1)
  //      -1b- se il file non e' specificato si visualizza un pixmap bianco
  //            di lato nullo
  //  -2- se larghezza e altezza sono specificate
  //      -2a- se il file e' specificato lo si visualizza con l'ingrandimento
  //            opportuno
  //      -2b- se il file non e' specificato si visualizza un pixmap bianco
  //            delle dimensioni richieste
  //  -3- non ci sono refresh impliciti
  void setFileTo(string fileName);
  bool redrawPs(unsigned requestedWidth=0,unsigned requestedHeight=0);
  static void setGhostscriptPathTo(string gsPath)
  { QPsWidget::_ghostscriptPath=gsPath; return; }
  static void setTimeoutTo(unsigned timeout)
  { QPsWidget::_timeout=timeout; return; }
};
