// page.C
// Copyright (C) 1999-2004 Paolo Ribeca

// This file is part of FeynCAD package.

// FeynCAD is free software;
// you can redistribute it and/or modify it under the terms
// of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with FeynCAD; see the file COPYING.
// If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <unistd.h>
#include <string>
#include <sstream>
#include <map>
#include "QFeynCADEditor.h"
#include "QPsWidget.h"
#include "QFeynCADCombined.h"

class page
{
  static unsigned _globalCounter;
protected:
  unsigned _progr;
  bool _isUntitled;
  QFeynCADCombined* _widget;
  string _title;
  string _tmpFileName;
  string _fileName;
public:
  page(QWidget* parent=0,string fileName=""):
    _progr(page::_globalCounter++),
    _isUntitled(fileName==string("")),
    _widget(new QFeynCADCombined(parent)),
    _fileName(fileName) {
    ostringstream tmpFileName;
    tmpFileName << QFeynCADEditor::_tmpPath << "/FeynCAD-" << getpid() << "-"
		<< this->_progr;
    this->_tmpFileName=tmpFileName.str();
    if (_isUntitled) {
      ostringstream title;
      title << "[Untitled-" << this->_progr << "]";
      this->_title=title.str();
    }
    else
      this->setFileNameTo(fileName);
    this->_widget->setBaseFileNameTo(this->_tmpFileName);
  }

  // Il distruttore dovrebbe fare pulizia di tutti i file temporanei
  ~page() { }

  void setFileNameTo(string fileName) {
    if (fileName==string(""))
      abort(); // EXCP
    // Nel caso si completa il nome
    if (fileName.size()<4||
	fileName[fileName.size()-4]!='.'||fileName[fileName.size()-3]!='f'||
	fileName[fileName.size()-2]!='c'||fileName[fileName.size()-1]!='d')
      fileName+=string(".fcd");
    this->_fileName=fileName;
    // Si trova l'ultima parte del nome per il titolo
    int where;
    for (where=fileName.size()-1;where>-1;--where)
      if (fileName[where]=='/')
	break;
    this->_title=string("");
    for (++where;where<(int)(fileName.size());++where)
      this->_title+=fileName[where];
    this->_isUntitled=false;
    return;
  }

  QFeynCADCombined* widget() const { return this->_widget; }
  string title(bool clean=false) const {
    if (this->_widget->_editor->_isUpToDate||clean)
      return this->_title;
    else {
      ostringstream provvTitle;
      provvTitle << "*" << this->_title;
      return provvTitle.str();
    }
  }
  string tmpFileName() const { return this->_tmpFileName; }
  string fileName() const { return this->_fileName; }

  inline bool isUntitled() { return this->_isUntitled; }
};
